/**
This class represent the default message generator for a <CODE>DiscussionGameHandler</CODE>.
Every argument in the framework is represented by a code:
<UL>
<LI>A, B, C, ... if the number of the arguments is equal or less than 26;
<LI>A1, B1, ..., A2, B2, ... otherwise.
</UL>
*/
public class DefaultMessageGenerator extends MessageGenerator
{
	private String[] myArgs;

	/**
	Construct a new <CODE>DefaultMessageGenerator</CODE> working on given arguments.
	@param args the arguments for generating new messages.
	*/
	public DefaultMessageGenerator(String[] args)
	{
		super(args);
	}

	/**
	Set the arguments on which the class work to a new set.
	@param args the arguments for generating new messages.
	*/
	public void setArgs(String[] args)
	{
		super.setArgs(args);
		init();
	}

	/**
	Returns the set of arguments used by this generator.
	@Return the set of arguments used by this generator
	*/
	public String[] getArgs()
	{
		return myArgs;
	}

	/*
	Initialize the generator by converting the arguments.
	*/
	private void init()
	{
		generateArgumentList(args.length);
	}

	/*
	Generates the new arguments' names.
	*/
	private void generateArgumentList(int n)
	{
		String suffix = "";
		boolean putSuffix = false;

		if (n > 26)
		{
			putSuffix = true;
			suffix = "1";
		}

		myArgs = new String[n];

		char nameChar = 'A';
		int charCount = 0;
		int numCount = 1;

		for (int i = 0; i < n; i++)
		{
			myArgs[i] = "" + nameChar + suffix;
			nameChar++;
			charCount++;

			if (charCount == 26)
			{
				numCount++;
				suffix = "" + numCount;
				charCount = 0;
			}
		}
	}

	/**
	Returns this generator's name of the argument specified in input, i.e. the argument
	having the specified index.
	@param argument the index of the argument which name has to be returned
	@return the name of the specified argument
	*/
	public String getArgumentName(int argument)
	{
		return myArgs[argument];
	}

	/**
	Returns a message generated accordingly to the specified parameters.
	@param type the type of the message
	@param game the type of the game played
	@param proponent whether this message is one of the proponent's
	@param argument the subject of the message
	@param previous the previous argument, if it has to be included in the message
	@return the message generated accordingly to the specified parameters
	*/
	public String getMessage(int type, int game, boolean proponent, int argument, int previous)
	{
		String msg = "";

		if (type == QUESTION_OUTPUT_G ||
			type == QUESTION_OUTPUT_NOTG_P ||
			type == QUESTION_OUTPUT_NOTP)
		{
			msg = "You have questioned me about the argument " + myArgs[argument] + ".\n";

			switch (type)
			{
				case QUESTION_OUTPUT_G: 	 msg += "Argument " + myArgs[argument] + " must be accepted.";
											 break;

				case QUESTION_OUTPUT_NOTG_P: msg += "Argument " + myArgs[argument] + " can be accepted, but it doesn't have to.";
											 break;

				case QUESTION_OUTPUT_NOTP: 	 msg += "Argument " + myArgs[argument] + " can't be accepted.";
											 break;

				default: error();
			}
		}

		else if (type == USER_DISAGREE)
		{
			switch (game)
			{
				case GROUNDED_GAME:	 if (proponent)
										msg = "I think that argument " + myArgs[argument] + " doesn't have to be always accepted.";
									 else
									 	msg = "I think that argument " + myArgs[argument] + " have to be accepted.";
									 break;
				case PREFERRED_GAME: if (proponent)
										msg = "I think that argument " + myArgs[argument] + " can't be accepted.";
									 else
									 	msg = "I think that argument " + myArgs[argument] + " can be accepted.";
									 break;
				default: error();
			}
		}

		else if (type == USER_ACCEPT)
			msg = "I don't have argument against it.";

		else if (type == START_DISCUSSION)
		{
			switch (game)
			{
				case GROUNDED_GAME:	 if (proponent)
										msg = "Let's discuss. I accept argument " + myArgs[argument] + "." ;
									 else
									 	msg = "You think that " + myArgs[previous] +
									 		  " must be accepted; but perhaps " + myArgs[previous] + "'s defeater " +
									 		  myArgs[argument] + " doesn't have to be rejected.";
									 break;
				case PREFERRED_GAME: if (proponent)
										msg = "Let's discuss. I think that " + myArgs[argument] + " can be accepted.";
									 else
									 	msg = "If you think that " + myArgs[previous] +
									 		  " is accepted, then you must hold that " + myArgs[previous] + "'s defeater " +
									 		  myArgs[argument] + " is rejected. Based on which ground do you reject " +
									 		  myArgs[argument] + "?";
									 break;
				default: error();
			}
		}

		else if (type == USER_SURRENDER)
			msg = "I don't have valid arguments anymore.";

		else if (type == PLAYING_GAME)
		{
			switch (game)
			{
				case GROUNDED_GAME:	 if (proponent)
										msg = "I reject " + myArgs[previous] + " because " +
											  myArgs[previous] + "'s defeater " + myArgs[argument] +
											  " must be accepted.";
									 else
									 	msg = "You think that " + myArgs[previous] +
									 		  " must be accepted; but perhaps " + myArgs[previous] + "'s defeater " +
									 		  myArgs[argument] + " doesn't have to be rejected.";
									 break;
				case PREFERRED_GAME: if (proponent)
										msg = "I reject " + myArgs[previous] + " because I accept "
											  + myArgs[previous] + "'s defeater " + myArgs[argument] + ".";
									 else
									 	msg = "If you think that " + myArgs[previous] +
											  " is accepted, then you must hold that " + myArgs[previous] + "'s defeater " +
											  myArgs[argument] + " is rejected. Based on which ground do you reject " +
									 		  myArgs[argument] + "?";
									 break;
				default: error();
			}

		}

		else if (type == GAME_OVER)
		{
			switch (game)
			{
				case GROUNDED_GAME:	 msg = "You cannot do more than repeating arguments.";
									 break;
				case PREFERRED_GAME: if (proponent)
										msg = "You are repeating yourself, we'd going around in a circle!";
									 else
									 	msg = "Your arguments are now contradicting!";
									 break;
				default: error();
			}
		}

		else if (type == WRONG_MOVE)
		{
			switch (game)
			{
				case GROUNDED_GAME:	 msg = "We've already discussed this point!";
									 break;
				case PREFERRED_GAME: if (proponent)
										msg = "This way you'd repeat yourself...";
									 else
									 	msg = "This way you will contradict yourself!";
									 break;
				default: error();
			}
		}

		else error();

		return msg;
	}

	private void error()
	{
		throw new IllegalArgumentException("The input for generating a message is wrong.");
	}
}