import argumentation.*;
import java.util.Scanner;

/**
The Discussion Game handler manages a discussion game and generate its output.
*/
public class DiscussionGameHandler
{
	private String[] args;					//the list of the arguments of the game.
	private boolean[][] def;				//the defeating relation between the arguments of the game
	private Scanner input;					//takes the input from the standard input.
	private ArgumentationFramework af;		//generates the extensions of the framework.
	private GroundedGamePlayer gp;			//plays the grounded game on the framework
	private PreferredGamePlayer pp;			//plays the preferred game on the framework
	private int gAnswer;					//answer in grounded game
	private int pAnswer;					//answer in preferred game
	private ArgumentationGamePlayer agp;	//currently used player
	private int currentGame;				//currently played game (from MessageGenerator)
	private boolean playingProponent;		//whether the program is playing as proponent
	private int currentArg;					//currently used argument
	private int previousArg;				//previously used argument
	private String[] userOptions;			//possible user answers - once known the choice, we can print the choosen one
	private int[] optionsMap;				//maps the sentences in userOptions to argument numbers
	private int questionCase;				//the result of the question on an argument (in {1, 2, 3})
	private DiscussionGameOutput out;		//the handler for game's output
	private MessageGenerator mesGen;		//the message generator

	/**
	Just build a new DiscussionGameHandler.
	*/
	public DiscussionGameHandler()
	{
		input = new Scanner(System.in);
	}

	/**
	Set the output handler for this <CODE>DiscussionGameHandler</CODE>.
	@param o the handler for the output of the game
	*/
	public void setOutput(DiscussionGameOutput o)
	{
		out = o;
	}

	/**
	Set the message generator for this <CODE>DiscussionGameHandler</CODE>.
	@param m the message generator for the output of the game
	*/
	public void setMessageGenerator(MessageGenerator m)
	{
		mesGen = m;
	}

	/**
	Loads a new framework from the file specified by the String in input,
	using a DefaultArgumentationFrameworkLoader.
	@param file the String specifying the path of the file.
	@throws FileNotFoundException if the file cannot be found.
	@throws ArgumentationFrameworkFormatException if the input file is
	not correctly formatted.
	*/
	public void load(String file)
	{
		load(new DefaultArgumentationFrameworkLoader(file));
	}

	/**
	Loads a new framework from ArgumentationFrameworkLoader.
	*/
	public void load(ArgumentationFrameworkLoader afl)
	{
		if (mesGen == null)
			mesGen = new DefaultMessageGenerator(afl.getArgs());
		else
			mesGen.setArgs(afl.getArgs());
		args = mesGen.getArgs();

		def = afl.getDef();
		af = new ArgumentationFramework(def);
		gp = new GroundedGamePlayer(def);
		pp = new PreferredGamePlayer(def);


		currentArg = -1;
		agp = null;
	}

	public String[] getArgs()
	{
		return args;
	}

	/**
	Returns the <CODE>ArgumentationFramework</CODE> for the current game's extensions management.
	@return the <CODE>ArgumentationFramework</CODE> for the current game's extensions management
	*/
	public ArgumentationFramework getExtensionManager()
	{
		return af;
	}

	/**
	Returns the currently used <CODE>MessageGenerator</CODE>
	@return the currently used <CODE>MessageGenerator</CODE>
	*/
	public MessageGenerator getMessageGenerator()
	{
		return mesGen;
	}

	/**
	Question an argument; the answer to the question can be one of the following:
	<OL>
	<LI> the argument must be accepted (that is, the argument is in the grounded extension of the framework,
	so no way it can be rejected);
	<LI> the argument can be accepted, but doesn't have to (that is, the argument is in some preferred
	extension of the game, so it's coherence can be proved in a preferred game even if it's not a
	gronded argument);
	<LI> the argument must be rejected (that is, it's neither in the grounded or in a preferred extension,
	so it's undefendable).
	</OL>
	*/
	public String[] question(int arg)
	{
		currentArg = arg;

		gAnswer = 0;
		pAnswer = 0;

		questionCase = 0;	//questionCase in {1, 2, 3}

		String msg = "";

		gp.init();
		gAnswer = gp.argue(arg);

		if (gAnswer == ArgumentationGamePlayer.PLAY_PROPONENT)
		{
			msg = mesGen.getMessage(MessageGenerator.QUESTION_OUTPUT_G, 0, arg);
			questionCase = 1;
			userOptions = new String[2];
			userOptions[1] = mesGen.getMessage(MessageGenerator.USER_DISAGREE, MessageGenerator.GROUNDED_GAME, true, arg, 0);
		}
		else
		{
			pp.init();
			pAnswer = pp.argue(arg);

			if (pAnswer == ArgumentationGamePlayer.PLAY_PROPONENT)
			{
				msg = mesGen.getMessage(MessageGenerator.QUESTION_OUTPUT_NOTG_P, 0, arg);
				questionCase = 2;
				userOptions = new String[3];
				userOptions[1] = mesGen.getMessage(MessageGenerator.USER_DISAGREE, MessageGenerator.PREFERRED_GAME, true, arg, 0);
				userOptions[2] = mesGen.getMessage(MessageGenerator.USER_DISAGREE, MessageGenerator.GROUNDED_GAME, false, arg, 0);
			}
			else
			{
				msg = mesGen.getMessage(MessageGenerator.QUESTION_OUTPUT_NOTP, 0, arg);
				questionCase = 3;
				userOptions = new String[2];
				userOptions[1] = mesGen.getMessage(MessageGenerator.USER_DISAGREE, MessageGenerator.PREFERRED_GAME, false, arg, 0);
			}
		}

		userOptions[0] = mesGen.getMessage(MessageGenerator.USER_ACCEPT);

		out.output(msg, DiscussionGameOutput.APPLICATION);

		return userOptions;
	}

	public String[] startDiscussion(int userAnswer)
	{
		out.output(userOptions[userAnswer], DiscussionGameOutput.USER);

		switch(questionCase)
		{
			case 1: agp = gp;	//DON'T HAVE to accept it - we've to play grounded on the argument, as proponent
					playingProponent = true;
					currentGame = MessageGenerator.GROUNDED_GAME;
					out.output(mesGen.getMessage(MessageGenerator.START_DISCUSSION, currentGame, playingProponent, currentArg, previousArg), DiscussionGameOutput.APPLICATION);
					break;
			case 2: if (userAnswer == 1)	//CAN'T accept - we've to play preferred on the argument, as proponent
					{
						agp = pp;
						playingProponent = true;
						currentGame = MessageGenerator.PREFERRED_GAME;
						out.output(mesGen.getMessage(MessageGenerator.START_DISCUSSION, currentGame, playingProponent, currentArg, previousArg), DiscussionGameOutput.APPLICATION);
					}
					else if (userAnswer == 2) //HAVE to accept - we've to play grounded on the argument, as opponent
					{
						previousArg = currentArg;
						currentArg = gAnswer;
						agp = gp;
						playingProponent = false;
						currentGame = MessageGenerator.GROUNDED_GAME;
						out.output(mesGen.getMessage(MessageGenerator.START_DISCUSSION, currentGame, playingProponent, currentArg, previousArg), DiscussionGameOutput.APPLICATION);
					}
					break;
			case 3: previousArg = currentArg;
					currentArg = pAnswer;
					agp = pp;		//CAN be accepted - we've to play preferred on the argument, as opponent
					playingProponent = false;
					currentGame = MessageGenerator.PREFERRED_GAME;
					out.output(mesGen.getMessage(MessageGenerator.START_DISCUSSION, currentGame, playingProponent, currentArg, previousArg), DiscussionGameOutput.APPLICATION);
					break;
		}

		userOptions = generateOptions(currentArg);

		return userOptions;
	}

	/**
	*/
	public String[] discuss(int userAnswer)
	{
		if (optionsMap[userAnswer] == def.length)	//the answer is not an argument, it's surrender!!
		{
			out.output(mesGen.getMessage(MessageGenerator.USER_SURRENDER), DiscussionGameOutput.USER);
			out.output("game over!", DiscussionGameOutput.SYSTEM);
			return null;
		}
		else
			out.output(mesGen.getMessage(MessageGenerator.PLAYING_GAME, currentGame, !playingProponent, optionsMap[userAnswer], currentArg), DiscussionGameOutput.USER);

		int newArg = agp.argue(optionsMap[userAnswer]); //in optionsMap there is the number of the argument which was that userAnswer

		if (newArg == ArgumentationGamePlayer.WRONG_MOVE)
		{
			if (!agp.hasNext())
			{
				gameOver();
				return null;
			}
			else
			{
				wrongMove();
				return userOptions;
			}
		}

		previousArg = currentArg;
		currentArg = newArg;

		out.output(mesGen.getMessage(MessageGenerator.PLAYING_GAME, currentGame, playingProponent, currentArg, previousArg), DiscussionGameOutput.APPLICATION);

		userOptions = generateOptions(currentArg);
		return userOptions;
	}


	private void gameOver()
	{
		out.output(mesGen.getMessage(MessageGenerator.GAME_OVER, currentGame, playingProponent), DiscussionGameOutput.APPLICATION);

		out.output("game over", DiscussionGameOutput.SYSTEM);
	}

	private void wrongMove()
	{
		out.output(mesGen.getMessage(MessageGenerator.WRONG_MOVE, currentGame, playingProponent), DiscussionGameOutput.APPLICATION);
	}


	private String[] generateOptions(int arg)
	{
		int numOpt = 1;

		for (int i = 0; i < def.length; i++)
			if (def[i][arg])
				numOpt++;

		String[] options = new String[numOpt];
		optionsMap = new int[numOpt];

		int optInd = 0;
		for (int i = 0; i < def.length; i++)
		{
			if (def[i][arg])
			{
				optionsMap[optInd] = i;
				options[optInd] = mesGen.getMessage(MessageGenerator.PLAYING_GAME, currentGame, !playingProponent, i, arg);
				optInd++;
			}
		}
		optionsMap[optInd] = def.length;
		options[optInd] = mesGen.getMessage(MessageGenerator.USER_SURRENDER);

		return options;
	}

	public String getGameExplanationMessage()
	{
		String message = "";

		if (agp == gp)		//If we're playing grounded game...
		{
			if (playingProponent)
				message = "The questioned argument was in the grounded extension\n" +
						  "of the framework.\n" +
						  "The user has asserted that the argument was not really\n" +
						  "grounded, so a grounded game has started, in which the\n" +
						  "user plays the role of the opponent.";
			else
				message = "The questioned argument wasn't in the grounded extension\n" +
						  "of the framework, thought it was in a preferred extension.\n" +
						  "The user has asserted that the argument IS actually grounded,\n" +
						  "so a grounded game has started, in which the user plays\n" +
						  "the role of the proponent.";
		}
		else if (agp == pp)	//else, if we're playing preferred game...
		{
			if (playingProponent)
				message = "The questioned argument was in the a preferred extension\n" +
						  "of the framework.\n" +
						  "The user has asserted that the argument wasn't really in\n" +
						  "such an extension, so a preferred game has started, in which\n" +
						  "the user plays the role of the opponent.";
			else
				message = "The questioned argument wasn't in neither the grounded or\n" +
						  "a preferred extension of the framework.\n" +
						  "The user has asserted that the argument was at least in a\n" +
						  "preferred extension, so a preferred game has started, in\n" +
						  "which the user plays the role of the proponent.";
		}

		return message;
	}
}