/**
A <CODE>MessageGenerator</CODE> generates the messages for a <CODE>DiscussionGameHandler</CODE>.
Extensions of this class can be used for modifying the appearence of the input and output of the game.
*/
public abstract class MessageGenerator
{
	/**
	Message type. The user accept the question's answer.
	*/
	public static final int USER_ACCEPT = -12;

	/**
	Message type. The user doesn't agree with the question's answer.
	*/
	public static final int USER_DISAGREE = - 1;

	/**
	Message type. The application propose to start a discussion.
	*/
	public static final int START_DISCUSSION = - 2;

	/**
	Message type. Message used during the game.
	*/
	public static final int PLAYING_GAME = - 13;

	/**
	Message type. Message used for user's wrong move.
	*/
	public static final int WRONG_MOVE = - 3;

	/**
	Message type. Message used for declaring game over.
	*/
	public static final int GAME_OVER = - 4;

	/**
	Message type. Output of the quesion, the argument has to be accepted.
	*/
	public static final int QUESTION_OUTPUT_G = - 6;

	/**
	Message type. Output of the quesion, the argument can be accepted, even if it doesn't have to.
	*/
	public static final int QUESTION_OUTPUT_NOTG_P = - 7;

	/**
	Message type. Output of the quesion, the argument has to be rejected.
	*/
	public static final int QUESTION_OUTPUT_NOTP = - 8;

	/**
	Message type. The user give it up.
	*/
	public static final int USER_SURRENDER = - 9;

	/**
	Game type. Grounded game.
	*/
	public static final int GROUNDED_GAME = - 10;

	/**
	Game type. Preferred game.
	*/
	public static final int PREFERRED_GAME = - 11;

	/**
	The set of arguments given to the generator.
	*/
	protected String[] args;

	/**
	Construct a new <CODE>MessageGenerator</CODE> with the given arguments as
	the set of arguments for generating messages.
	@param args the set of arguments to be used for generating new messages
	*/
	public MessageGenerator(String[] args)
	{
		this.args = args;
	}

	/**
	Returns the name of the specified argument in this generator.
	@param argument the index of the argument
	@return the name of the specified argument in this generator
	*/
	public abstract String getArgumentName(int argument);

	/**
	Returns the set of arguments used by this generator.
	@Return the set of arguments used by this generator
	*/
	public abstract String[] getArgs();

	/**
	Set the arguments used by this generator to the specified set.
	@param args the set of arguments to be used for further messages
	*/
	public void setArgs(String[] args)
	{
		this.args = args;
	}


	/**
	Returns a message generated accordingly to the specified parameters.
	The non specified ones are should be meaningless.
	@param type the type of the message
	@return the message generated accordingly to the specified parameters
	*/
	public String getMessage(int type)
	{
		return getMessage(type, 0, false, 0, 0);
	}

	/**
	Returns a message generated accordingly to the specified parameters.
	The non specified ones are should be meaningless.
	@param type the type of the message
	@param game the type of the game played
	@return the message generated accordingly to the specified parameters
	*/
	public String getMessage(int type, int game)
	{
		return getMessage(type, game, false, 0, 0);
	}

	/**
	Returns a message generated accordingly to the specified parameters.
	The non specified ones are should be meaningless.
	@param type the type of the message
	@param game the type of the game played
	@param proponent whether this message is one of the proponent's
	*/
	public String getMessage(int type, int game, boolean proponent)
	{
		return getMessage(type, game, proponent, 0, 0);
	}


	/**
	Returns a message generated accordingly to the specified parameters.
	The non specified ones are should be meaningless.
	@param type the type of the message
	@param game the type of the game played
	@param argument the subject of the message
	*/
	public String getMessage(int type, int game, int argument)
	{
		return getMessage(type, game, true, argument, 0);
	}


	/**
	Returns a message generated accordingly to the specified parameters.
	@param type the type of the message
	@param game the type of the game played
	@param proponent whether this message is one of the proponent's
	@param argument the subject of the message
	@param previous the previous argument, if it has to be included in the message
	@return the message generated accordingly to the specified parameters
	*/
	public abstract String getMessage(int type, int game, boolean proponent, int argument, int previous);
}