package argumentation;

import java.util.Iterator;

/**
An argumentation framework is a pair (Ar, def) where Ar is a set of arguments,
and def is a relation between arguments.
In this implementation the relation def is represented by a boolean squared n-dimensional matrix def[][],
such that
- n is the number of the arguments of the framework, and
- argument n1 defeats argument n2 iff def[n1][n2] = true.
The set of n arguments of the framework is represented by the indexes {0, 1, ..., n-1} of the def matrix.
*/
public class ArgumentationFramework
{
	private boolean def[][];					//def[a] = array of arguments defeated by a

	private LabellingSet potentialSemiStable;	//potential-semi-stable set
	private LabellingSet potentialStable;		//potential-stable set
	private LabellingSet potentialPreferred;	//potential-preferred set


	/**
	Construct a new Argumentation Framework.
	@param def a boolean matrix representing the defeating relation between arguments, such that
	def[a][b] is true iff argument a defeats argument b.
	*/
	public ArgumentationFramework(boolean[][] def)
	{
		this.def = def;

		potentialSemiStable = new LabellingSet();
		potentialStable = new LabellingSet();
		potentialPreferred = new LabellingSet();

	}

	/**
	Returns the defeating relation of the framework.
	@return the defeating relation of the framework
	*/
	public boolean[][] getDef()
	{
		return def;
	}


	//EXTENSION COMPUTING FUNCTIONS

	/*
	Returns the grounded extension as an array of booleans args[], where args[n] = true
	iff the n-th argument is in the 'in' set of the grounded extension.
	@return the grounded extension of the argumentation framework
	*/
	public Extension getGroundedExtension()
	{
		boolean[] extension = new boolean[def.length];
		boolean[] defendedByExt = defendedBy(extension);

		while (!equals(extension, defendedByExt))
		{
			extension = merge(extension, defendedByExt);
			defendedByExt = defendedBy(defendedByExt);
		}

		return new Extension(extension);
	}

	/**
	Returns the preferred extensions of the framework as a set of labellings, i.e. a set of
	arrays of integers. Each labelling is an array of integers a[] such that
	- a[i] = IN iff the i-th argument is 'in' in the labelling
	- a[i] = OUT iff the i-th argument is 'out' in the labelling
	- a[i] = UNDEC iff the i-th argument is 'undec' in the labelling
	@return the set of preferred extensions of the argumentation framework
	*/
	public Extension[] getPreferredExtensions()
	{
		findPreferred(Labelling.allINLabelling(def.length));
		return labellingSetToExtensions(potentialPreferred);
	}

	/**
	Returns the stable extensions of the framework as a set of labellings, i.e. a set of
	arrays of integers. Each labelling is an array of integers a[] such that
	- a[i] = IN iff the i-th argument is 'in' in the labelling
	- a[i] = OUT iff the i-th argument is 'out' in the labelling
	- a[i] = UNDEC iff the i-th argument is 'undec' in the labelling
	@return the set of stable extensions of the argumentation framework, or null if such an extension doesn't exist
	*/
	public Extension[] getStableExtensions()
	{
		findStable(Labelling.allINLabelling(def.length));
		return labellingSetToExtensions(potentialStable);
	}

	/**
	Returns the semi-stable extensions of the framework as a set of labellings, i.e. a set of
	arrays of integers. Each labelling is an array of integers a[] such that
	- a[i] = IN iff the i-th argument is 'in' in the labelling
	- a[i] = OUT iff the i-th argument is 'out' in the labelling
	- a[i] = UNDEC iff the i-th argument is 'undec' in the labelling
	@return the set of semi-stable extensions of the argumentation framework
	*/
	public Extension[] getSemiStableExtensions()
	{
		findSemiStable(Labelling.allINLabelling(def.length));
		return labellingSetToExtensions(potentialSemiStable);
	}


	//END EXTENSION COMPUTING FUNCTIONS



	//EXTENSION COMPUTING ALGORITHMS

	/*
	This algorithm finds all semi-stable extensions and store them in the set potentialSemiStable.
	*/
	private void findSemiStable(Labelling labelling)
	{
		Iterator iterator = potentialSemiStable.iterator();

		//if we have something worse than found earlier, then prune and backtrack
		while (iterator.hasNext())
		{
			if (labelling.isSuperset(((Labelling)iterator.next()), Labelling.UNDEC))
				return;
		}

		//now see if the transition sequence has terminated
		if (!hasIllegallyIn(labelling))
		{
			iterator = potentialSemiStable.iterator();

			while (iterator.hasNext())
			{
				//if old result is worse than the new labelling, remove it
				if (((Labelling)iterator.next()).isSuperset(labelling, Labelling.UNDEC))
					iterator.remove();
			}

			//add our newly found labelling as a candidate; we already know
			//that it is not worse than what we already have
			potentialSemiStable.add(labelling);

			return;
		}
		else
		{
			int arg = getSuperIllegallyIn(labelling);

			if (arg >= 0)
				findSemiStable(transitionStep(arg, labelling));
			else
				for (int i = 0; i < labelling.labels.length; i++)
					if (isIllegallyIn(i, labelling))
						findSemiStable(transitionStep(i, labelling));
		}
	}


	/*
	This algorithm finds all preferred extensions and store them in the set potentialPreferred.
	*/
	private void findPreferred(Labelling labelling)
	{
		Iterator iterator = potentialPreferred.iterator();

		//if we have something worse than found earlier, then prune and backtrack
		while (iterator.hasNext())
		{
			if (((Labelling)iterator.next()).isSuperset(labelling, Labelling.IN))
				return;
		}

		//now see if the transition sequence has terminated
		if (!hasIllegallyIn(labelling))
		{
			iterator = potentialPreferred.iterator();

			while (iterator.hasNext())
			{
				//if old result is worse than the new labelling, remove it
				if (labelling.isSuperset(((Labelling)iterator.next()), Labelling.IN))
					iterator.remove();
			}

			//add our newly found labelling as a candidate; we already know
			//that it is not worse than what we already have
			potentialPreferred.add(labelling);

			return;
		}
		else
		{
			int arg = getSuperIllegallyIn(labelling);

			if (arg >= 0)
				findPreferred(transitionStep(arg, labelling));
			else
				for (int i = 0; i < labelling.labels.length; i++)
					if (isIllegallyIn(i, labelling))
						findPreferred(transitionStep(i, labelling));
		}
	}


	/*
	This algorithm finds all stable extensions and store them in the set potentialSemiStable.
	*/
	private void findStable(Labelling labelling)
	{
		Iterator iterator = potentialStable.iterator();

		//if the current labelling has undecided, then prune and backtrack
		if (labelling.contains(Labelling.UNDEC))
			return;

		//now see if the transition sequence has terminated
		if (!hasIllegallyIn(labelling))
		{
			//add our newly found labelling as a candidate; we already know
			//that it is not worse than what we already have
			potentialStable.add(labelling);

			return;
		}
		else
		{
			int arg = getSuperIllegallyIn(labelling);

			if (arg >= 0)
				findStable(transitionStep(arg, labelling));
			else
				for (int i = 0; i < labelling.labels.length; i++)
					if (isIllegallyIn(i, labelling))
						findStable(transitionStep(i, labelling));
		}
	}


	/*
	TRANSITION STEP:
	1) the label of A is changed from in to out
	2) for all B belonging to {A}UA+, if B is illegally out change B from out to undec

	The new labelling is returned.
	*/
	private Labelling transitionStep(int arg, Labelling labelling)
	{
		Labelling newLabelling = labelling.clone();

		newLabelling.labels[arg] = Labelling.OUT;

		if (isIllegallyOut(arg, newLabelling))
			newLabelling.labels[arg] = Labelling.UNDEC;

		for (int i = 0; i < def.length; i++)
			if (def[arg][i] && isIllegallyOut(i, newLabelling))
				newLabelling.labels[i] = Labelling.UNDEC;

		return newLabelling;
	}

	//END EXTENSION COMPUTING ALGORITHMS



	/*
	Returns true iff the given labelling contains an "illegally in" argument.
	*/
	private boolean hasIllegallyIn(Labelling l)
	{
		for (int i = 0; i < l.labels.length; i++)
			if (isIllegallyIn(i, l))
				return true;
		return false;
	}

	/*
	Returns the index of a "super-illegally in" argument with respect to the given labelling,
	or -1 if such an argument doesn't exixts.
	*/
	private int getSuperIllegallyIn(Labelling l)
	{
		for (int i = 0; i < l.labels.length; i++)
			if (isSuperIllegallyIn(i, l))
				return i;
		return -1;
	}

	/**
	Check whether an argument is "illegally in" in a given labelling.
	@param arg the index of the argument to be checked for illegality
	@param l the label with regards to the argument has to be checked
	@return true if the argument is "illegally in" in the given labelling, false otherwise.
	@throws IllegalArgumentException if the given labelling is not compatible with the set of arguments of
	this framework, i.e. the number of the arguments is different.
	*/
	private boolean isIllegallyIn(int arg, Labelling l)
	{
		if (l.labels.length != def.length) throw new IllegalArgumentException("Non compatible labelling.");

		if (l.labels[arg] != Labelling.IN)
			return false;

		for (int i = 0; i < l.labels.length; i++)
			if (def[i][arg] && !(l.labels[i] == Labelling.OUT))
				return true;
		return false;
	}

	/**
	Check whether an argument is "superillegally in"in a given labelling.
	@param arg the index of the argument to be checked for illegality
	@param l the label with regards to the argument has to be checked
	@return true if the argument is "superillegally in" in the given labelling, false otherwise.
	@throws IllegalArgumentException if the given labelling is not compatible with the set of arguments of
	this framework, i.e. the number of the arguments is different.
	*/
	private boolean isSuperIllegallyIn(int arg, Labelling l)
	{
		if (l.labels.length != def.length) throw new IllegalArgumentException("Non compatible labelling.");

		if (l.labels[arg] != Labelling.IN)
			return false;

		for (int i = 0; i < l.labels.length; i++)
			if (def[i][arg] &&
					(l.labels[i] == Labelling.UNDEC ||
					   (l.labels[i] == Labelling.IN &&  !(isIllegallyIn(i, l) || isSuperIllegallyIn(i, l)))
					)
			   )
				return true;
		return false;
	}

	/**
	Check whether an argument is "illegally out" in a given labelling.
	@param arg the index of the argument to be checked for illegality
	@param l the label with regards to the argument has to be checked
	@return true if the argument is "illegally out" in the given labelling, false otherwise.
	@throws IllegalArgumentException if the given labelling is not compatible with the set of arguments of
	this framework, i.e. the number of the arguments is different.
	*/
	private boolean isIllegallyOut(int arg, Labelling l)
	{
		if (l.labels.length != def.length) throw new IllegalArgumentException("Non compatible labelling.");

		if (l.labels[arg] != Labelling.OUT)
			return false;

		for (int i = 0; i < l.labels.length; i++)
			if (def[i][arg] && (l.labels[i] == Labelling.IN))
				return false;
		return true;
	}

	/**
	Check whether an argument is "illegally undecided" in a given labelling.
	@param arg the index of the argument to be checked for illegality
	@param l the label with regards to the argument has to be checked
	@return true if the argument is "illegally undecided" in the given labelling, false otherwise.
	@throws IllegalArgumentException if the given labelling is not compatible with the set of arguments of
	this framework, i.e. the number of the arguments is different.
	*/
	private boolean isIllegallyUndec(int arg, Labelling l)
	{
		if (l.labels.length != def.length) throw new IllegalArgumentException("Non compatible labelling.");

		if (l.labels[arg] != Labelling.UNDEC)
			return false;

		boolean canBeUndec = false;
		for (int i = 0; i < l.labels.length; i++)
		{
			if (def[i][arg])
			{
				if (l.labels[i] == Labelling.IN)
					return false;
				else if (l.labels[i] == Labelling.UNDEC)
					canBeUndec = true;
			}
		}
		return canBeUndec;
	}

	/**
	Checks whether the given labelling is complete in this Argumentation Framework.
	@param the labelling to be checked for completeness
	@return true if the labelling is a complete labelling, false otherwise
	@throws IllegalArgumentException if the given labelling is not compatible with the set of arguments of
	this framework, i.e. the number of the arguments is different.
	*/
	public boolean isComplete(Labelling l)
	{
		if (l.labels.length != def.length) throw new IllegalArgumentException("Non compatible labelling.");

		for (int i = 0; i < l.labels.length; i++)
			if (isIllegallyIn(i, l) || isIllegallyOut(i, l) || isIllegallyUndec(i, l))
				return false;
		return true;
	}


	/*
	Function F(...)
	Returns a set of arguments defended by the set of arguments given as input.
	A = defendedBy(B) iff A = F(B).
	*/
	private boolean[] defendedBy(boolean[] a)
	{
		boolean[] defeated = defeatedBy(a);					//defeated = a+
		boolean[] defended = new boolean[def.length];

		for (int i = 0; i < a.length; i++)
		{
			boolean[] defeaters = defeaters(i);
			if (isIncluded(defeaters, defeated))
				defended[i] = true;
		}

		return defended;
	}

	/*
	Set {...}+
	Returns a set of arguments defeated by the set of arguments given as input.
	A = defeatedBy(B) iff A = B+
	*/
	private boolean[] defeatedBy(boolean[] a)
	{
		boolean[] defeatedArgs = new boolean[def.length];

		for (int i = 0; i < a.length; i++)
			if (a[i])
				defeatedArgs = merge(defeatedArgs, def[i]);

		return defeatedArgs;
	}


	/*
	Set A-
	Returns the set of defeaters of the argument given as input.
	B is in defeaters(A) iff B is in A-
	*/
	private boolean[] defeaters(int a)
	{
		boolean[] defeaters = new boolean[def.length];
		for (int i = 0; i < defeaters.length; i++)
			if (def[i][a])
				defeaters[i] = true;
		return defeaters;
	}


	private Extension labellingToExtension(Labelling l)
	{
		boolean[] ext = new boolean[l.labels.length];
		for (int i = 0; i < l.labels.length; i++)
			if (l.labels[i] == Labelling.IN)
				ext[i] = true;
		return new Extension(ext);
	}

	private Extension[] labellingSetToExtensions(LabellingSet ls)
	{
		Iterator i = ls.iterator();
		Extension[] exts = new Extension[ls.size()];

		for (int j = 0; i.hasNext(); j++)
			exts[j] = labellingToExtension((Labelling)i.next());

		return exts;
	}

	/*
	Convert a labelling in the form of an array of boolean into an array of integers.
	input = array 'in' of boolean
	output = a[] of integers, such that
		a[i] = IN iff in[i] = true,
		a[i] = OUT iff i belongs to in+,
		a[i] = UNDEC otherwise.
	*/
	private int[] booleanToLabelling(boolean[] in)
	{
		boolean[] out = defeatedBy(in);

		int[] labelling = new int[def.length];

		for (int i = 0; i < labelling.length; i++)
		{
			if (in[i])
				labelling[i] = Labelling.IN;
			else if (out[i])
				labelling[i] = Labelling.OUT;
			else
				labelling[i] = Labelling.UNDEC;
		}
		return labelling;
	}



	//SOME BASIC SET OPERATIONS

	/*
	Inclusion
	isIncluded(a, b) = TRUE iff (a belongs to b)
	*/
	private boolean isIncluded(boolean[] a, boolean[] b)
	{
		for (int i = 0; i < a.length; i++)
			if (a[i] && !b[i])
				return false;
		return true;
	}

	/*
	Union
	merge(a, b) = a union b
	*/
	private boolean[] merge(boolean[] a, boolean[] b)
	{
		boolean[] m = new boolean[def.length];

		for (int i = 0; i < a.length; i++)
			m[i] = (a[i] || b[i]);

		return m;
	}

	/*
	Equality
	equals(a, b) = TRUE iff (an element x belongs to a iff x belongs to b)
	*/
	private boolean equals(boolean[] a, boolean[] b)
	{
		for (int i = 0; i < a.length; i++)
			if (a[i] != b[i])
				return false;
		return true;
	}

	//END OF BASIC SET OPERATIONS

}

