package argumentation;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;

import java.util.StringTokenizer;
import java.util.List;
import java.util.LinkedList;

import java.io.FileNotFoundException;
import java.io.IOException;

import java.util.Iterator;
import java.util.NoSuchElementException;


/**
The DefaultArgumentationFrameworkLoader loads an argumentation framework from
a text file.
The file should be formatted as follows:
...
0-n empty lines
...
	[list of the m arguments of the framework, as a
	 list of m sentences, one for each line]
...
1-n empty lines
...
	[defeating relation between arguments, represented
	 as couple of integers "a b", both in 1..m; the couple
	 <a, b> is in the list iff argument a defeats argument b]
...
0-n empty lines
...

Example of a correctly formatted input file:

 -----------begin-----------
|
|the cat is blue
|a cat can be easily painted
|it doesn't exist a blue cat
|painting a cat is not easy at all
|
|
|2 3
|3 1
|4 2
|
 ------------end------------

*/
public class DefaultArgumentationFrameworkLoader implements ArgumentationFrameworkLoader
{
	String[] args;		//the list of the arguments of the loaded framework
	boolean[][] def;	//the defeating relation between the arguments of the loaded framework

	boolean frameworkLoaded;	//whether or not a framework has been loaded

	/**
	Constructs a new empty DefaultArgumentationFrameworkLoader.
	*/
	public DefaultArgumentationFrameworkLoader()
	{
		frameworkLoaded = false;
	}

	/**
	Constructs a new <CODE>DefaultArgumentationFrameworkLoader</CODE>, loading the given file.
	@param filename the path of the input file
	@throws ArgumentationFrameworkException if the file is not correctly formatted.
	*/
	public DefaultArgumentationFrameworkLoader(String filename) throws ArgumentationFrameworkFormatException
	{
		this();
		loadFramework(filename);
	}

	/**
	Load a new argumentation framework from the given file.
	@param filename the path of the input file
	@throws ArgumentationFrameworkException if the file is not correctly formatted.
	*/
	public void loadFramework(String filename) throws ArgumentationFrameworkFormatException
	{
		String errorMessage = null;
		int lineCount = 0;
		try
		{
			BufferedReader input = new BufferedReader(new InputStreamReader(new FileInputStream(filename)));

			//REDING ARGUMENTS SET
			List argList = new LinkedList();	//List for temporary store arguments

			String inputLine = "";

			while (inputLine != null && inputLine.equals(""))	//don't consider empty lines
			{
				inputLine = input.readLine().trim();
				lineCount++;
			}

			while (inputLine != null && !inputLine.trim().equals(""))	//found a non-empty line, read arguments
			{
				argList.add(inputLine);
				inputLine = input.readLine();
				lineCount++;
			}

			//END REDING ARGUMENTS SET


			args = (String[])argList.toArray(new String[0]);
			def = new boolean[args.length][args.length];

			while (inputLine != null && inputLine.equals(""))	//don't consider empty lines again
			{
				inputLine = input.readLine().trim();
				lineCount++;
			}

			int defeater;
			int defeated;

			//REDING DEFEATING ARROWS
			while (inputLine != null && !inputLine.trim().equals(""))	//read couples defeater/defeated
			{
				StringTokenizer st = new StringTokenizer(inputLine);
				defeater = Integer.parseInt(st.nextToken()) - 1;
				defeated = Integer.parseInt(st.nextToken()) - 1;
				def[defeater][defeated] = true;

				inputLine = input.readLine();
				lineCount++;
			}
			//END REDING DEFEATING ARROWS
		}
		catch (FileNotFoundException e)	{	errorMessage = "File not found.";	}
		catch (NullPointerException e)	{	errorMessage = "The input file is not correctly formatted (line " + lineCount + ").\nPlease check the definition of the defeating relation between arguments,\nprobably some index is wrong.";	}
		catch (NumberFormatException e)	{	errorMessage = "The input file is not correctly formatted (line " + lineCount + ").\nPlease check the definition of the defeating relation between arguments,\nprobably some index is not a number.";	}
		catch (NoSuchElementException e){	errorMessage = "The input file is not correctly formatted (line " + lineCount + ").\nPlease check the definition of the defeating relation between arguments,\nprobably an element of some couple is missing.";	}
		catch (IOException e)			{	errorMessage = "An error has occurred while reading the input file.";	}
		finally
		{
			if (errorMessage != null)
				throw new ArgumentationFrameworkFormatException(errorMessage);
		}
	}

	/**
	Returns the set of loaded arguments.
	@return the set of loaded arguments
	*/
	public String[] getArgs()
	{
		return args;
	}

	/**
	Returns the loaded defeating relation.
	@return the loaded defeating relation
	*/
	public boolean[][] getDef()
	{
		return def;
	}
}