package argumentation;

/**
A labelling is a label assignment to a set of arguments such that each argument has a label in {IN, OUT, UNDEC}.
Please note that in this implementation label assignment is on the position of the argument, not on the name, so
the coherence between arguments and labels must been kept by the user.
*/
public class Labelling
{
	//DEFINITION OF THE VALUES FOR THE 3 LABELS; NOTE THAT ON THOSE
	//VALUES IS BASED THE WAY THE LabellingSet CLASS WORKS
	/**
	The argument is IN.
	*/
	public static final int IN = 1;

	/**
	The argument is OUT.
	*/
	public static final int OUT = 0;

	/**
	The argument is UNDECIDED.
	*/
	public static final int UNDEC = 2;

	public int[] labels;			//the array of labels for this labelling.

	/**
	Builds a new labelling from the labels passed as parameters. Values for the labelling must be in {IN, OUT, UNDEC}.
	@param labels the labels for this labelling.
	*/
	public Labelling(int[] labels)
	{
		for (int i = 0; i < labels.length; i++)
			if (labels[i] > 2 || labels[i] < 0)
				throw new IllegalArgumentException("The labels are not in {IN, OUT, UNDEC}");
		this.labels = labels;
	}

	/**
	Compares a given labelling to this labelling for equality.
	@param labelling the labelling to be compared with this
	@return true if the two labellings are equals, false otherwise
	*/
	public boolean equals(Labelling l)
	{
		if (l.labels.length != this.labels.length) throw new IllegalArgumentException("Trying to compare two incompatible labellings.");

		for (int i = 0; i < labels.length; i++)
			if (l.labels[i] != labels[i])
				return false;
		return true;
	}

	/**
	Check whether this labelling is a superset of a given labelling with regard to the given label.
	For example, if the given label is IN, this method checks wheter or not this labelling's in() set is
	a superset of the given labelling's one.
	@param labelling the labelling to compare with this labelling
	@param label the label with regard to checking superset relation
	@return true if this labelling is a superset of the given one with regard to the given label, false otherwise
	*/
	public boolean isSuperset(Labelling l, int label)
	{
		if (l.labels.length != this.labels.length) throw new IllegalArgumentException("Trying to compare two incompatible labellings.");

		boolean isProperSuperset = false;

		for (int i = 0; i < labels.length; i++)
		{
			if (!(labels[i] == label) && l.labels[i] == label)
				return false;
			if (!isProperSuperset && labels[i] == label && !(l.labels[i] == label))
				isProperSuperset = true;
		}
		return isProperSuperset;
	}


	/**
	Returns true iff the set of elements equal to the given label is not empty
	@return true if the set of elements equal to the given label is not empty, false otherwise
	*/
	public boolean contains(int label)
	{
		for (int i = 0; i < labels.length; i++)
			if (labels[i] == label)
				return true;
		return false;
	}

	/**
	Returns an array of boolean representing the subset of arguments having the specified label,
	such that an element of the array is true iff the corresponding argument has the specified
	label.
	@return an array of boolean representing the subset of arguments having the specified label
	*/
	public boolean[] getSubset(int label)
	{
		boolean[] output = new boolean[labels.length];
		for (int i = 0; i < labels.length; i++)
			if (labels[i] == label)
				output[i] = true;
		return output;

	}


	public Labelling clone()
	{
		int[] labs = new int[labels.length];
		for (int i = 0; i < labels.length; i++)
			labs[i] = labels[i];
		return new Labelling(labs);
	}

	/**
	Returns a new labelling in which all arguments are 'in'.
	@param n the number of the arguments in the new labelling.
	@return a new labelling in which all arguments are 'in'
	*/
	public static Labelling allINLabelling(int n)
	{
		int[] l = new int[n];
		for (int i = 0; i < l.length; i++)
			l[i] = IN;

		return new Labelling(l);
	}

}