package argumentation;

import java.util.Iterator;

/**
This class implements a set for labellings. It accepts instances of the class Labelling.
The internal representation of the set consists in a 3-branched tree, in which every
branch can represent either IN or OUT or UNDEC, and every path from the root to a leaf
is a labelling.
The class provides also an iterator on the set of labellings, implementing the Iterator
interface in java.util.
*/
public class LabellingSet
{
	private int argNum;		//number of labelled arguments
	private Node root;

	/**
	Contstruct a new empty LabellingSet.
	*/
	public LabellingSet()
	{
		argNum = 0;			//the set is empty
		root = new Node();	//the root is a new node without children
	}

	/**
	Adds a new labelling to the set.
	@param labelling the labelling to be added to the set.
	@return true if the labelling has been inserted, false if the labelling already exists in the set or if it is empty.
	@throws IllegalArgumentException if the input labelling is not well-formed. A labelling is well-formed
	if for each element l, 0 <= l <= 2, and if the length of the labelling is equal to the length of all
	labellings already put into this set.
	*/
	public boolean add(Labelling labelling)
	{
		if (argNum == 0)					//if the number of arguments hasn't been set yet, use the labelling's length
			argNum = labelling.labels.length;
		else if (labelling.labels.length != argNum)
			throw new IllegalArgumentException("Illegally trying to add a " +
												labelling.labels.length + "-elements labelling to a " +
												argNum + "-elements labelling set!");
		return add(root, labelling.labels, 0);
	}

	/*
	This method performs the addition to the set; it tries to add the elements from position 'i' to the end of the
	labelling to the given node of the tree.
	*/
	private boolean add(Node node, int[] lab, int i)
	{
		if (i == argNum) return false;	//we are in a leaf, no more elements to be added

		if (node.next[lab[i]] == null)	//if the path corresponding to the label doesn't exist, create the next node
		{
			node.next[lab[i]] = new Node();
			add(node.next[lab[i]], lab, ++i);
		}
		else if (!add(node.next[lab[i]], lab, ++i))			//next node already exists, try to add the remain
			return false;									//if it won't add anything new, return false

		node.paths++;				//something new has been added, increase number of paths and return true
		return true;
	}


	/**
	Returns the cardinality of this set.
	@return the cardinality of this set
	*/
	public int size()
	{
		return root.paths;
	}

	/**
	Returns an iterator for this set of labellings.
	@return an iterator for this set of labellings
	*/
	public Iterator iterator()
	{
		return new LabellingSetIterator();
	}


	public Labelling[] toArray()
	{
		Labelling[] ll = new Labelling[root.paths];
		Iterator i = iterator();

		for (int j = 0; i.hasNext(); j++)
		{
			ll[j] = (Labelling)i.next();
		}

		return ll;
	}

	/*
	This  class implements a node of the 3-branched tree used as a set of labellings.
	-  next is a 3-elements array of nodes containing the branches of the node
	-  paths is the number of paths of the subtree which has this node as root; i.e., the number of
	   labellings contained by the subtree itself
	*/
	private class Node
	{
		public int paths;
		public Node[] next;

		public Node()
		{
			paths = 0;
			next = new Node[3];
		}
	}



		//LABELLING SET ITERATOR
		/*
		Implements an iterator over a LabellingSet. This class implements the interface Iterator in order to
		have a publicly accessible interface.
		*/
		private class LabellingSetIterator implements Iterator
		{
			private boolean removeFlag;			//according to the definition of the interface Iterator, it's possible to remove
												//an object by means of the iterator only if next() has been previously called

			private boolean lookedAhead;		//lookedAhead = true iff the set has been checked for more elements

			private boolean isActive;			//the iterator is active if it hasn't been found that there are no
												//more elements to iterate on.

			private boolean newIterator;		//newIterator is true at the beginning, when the first call to method next()
												//have to find out the first labelling and build the backtrackNodes array

			private Node[] backtrackNodes;		//keeps the current path's nodes for backtracking

			private int[] currentLabelling;		//the last returned labelling; it is kept for doing backtracking by
												//trying to increase the value of the labels

			/*
			Build a new LabellingSetIterator.
			*/
			public LabellingSetIterator()
			{
				if (root.paths == 0)
					isActive = false;
				else
				{
					removeFlag = false;
					isActive = true;			//sure it has next,
					lookedAhead = true;			//because it's not empty
					newIterator = true;
					currentLabelling = new int[argNum];
					backtrackNodes = new Node[argNum];
				}
			}

			/*
			Returns the next labelling in the set.
			*/
			public Object next()
			{
				if (!hasNext())
					throw new java.util.NoSuchElementException("No more labellings in the set.");

				getNextLabelling();

				removeFlag = true;

				return new Labelling(currentLabelling);
			}

			/*
			Returns true only if exists another labelling in the set.
			*/
			public boolean hasNext()
			{
				if (isActive && !lookedAhead)
				{
					isActive = lookForMore();
					lookedAhead = true;
				}
				return isActive;
			}

			/*
			Check the set of backtrack nodes for more unexplored paths (not null pointers); returns
			true if such a path is found.
			*/
			private boolean lookForMore()
			{
				if (newIterator)
					return true;

				if (isActive)
					for (int i = 0; i < backtrackNodes.length; i++)		//for every node in the backtrack set, if there is some unexplored branch return true.
						for (int j = currentLabelling[i] + 1; j < 3; j++)
							if (backtrackNodes[i].next[j] != null)
								return true;

				isActive = false;
				return false;
			}

			/*
			Remove the last returned labelling
			*/
			public void remove()
			{
				if (!removeFlag) throw new IllegalStateException("next() method not previously called.");

				for (int i = backtrackNodes.length - 1; i >= 0; i--)		//remove the current labelling's leaf, and then check whether internal nodes have to be deleted.
				{
					if (backtrackNodes[i].next[currentLabelling[i]].paths == 0)
						backtrackNodes[i].next[currentLabelling[i]] = null;

					backtrackNodes[i].paths--;
				}
			}


			/*
			Gets the next labelling, even backtracking through the tree
			*/
			private void getNextLabelling()
			{
				if (newIterator)
				{
					buildNextLabelling(root, 0);
					newIterator = false;
				}
				else
					searchForNextLabelling(argNum - 1);

				lookedAhead = false;
			}

			/*
			This method searchs for the next labelling performing a backtrack on the tree, and once found a fruitful backtrack point
			it calls buildNextLabelling for going forward in the tree until the first available leaf.
			*/
			private void searchForNextLabelling(int i)
			{
				currentLabelling[i]++;		//try to explore another branch

				while (currentLabelling[i] < 3)
				{
					if (backtrackNodes[i].next[currentLabelling[i]] != null)	//There is an unexplored branch, build next labelling!
					{
						buildNextLabelling(backtrackNodes[i].next[currentLabelling[i]], i + 1);
						return;
					}
					else
						currentLabelling[i]++;
				}

				searchForNextLabelling(i - 1);		//couldn't find another branch, let's try to backtrack more...
			}


			/*
			Thi method goes deep in the first available path, recursively until the leaf; this way it find next labelling after a backtrack.
			*/
			private void buildNextLabelling(Node node, int i)
			{
				if (i == argNum)		//we're at the leaf
					return;

				for (int j = 0; j < 3; j++)
				{
					if (node.next[j] != null)
					{
						currentLabelling[i] = j;
						backtrackNodes[i] = node;
						buildNextLabelling(node.next[j], i + 1);
						return;
					}
				}
			}
		}

		//END OF LABELLING SET ITERATOR
}